
/*================================================*\

  Vendredi 4 decembre 1998

  Arash HABIBI
  
  Detection.c

  Ce qu'il faut pour calculer le CRC de n'importe
  quelle structure et ensuite, etant donnee 
  n'importe quelle structure de voir si le 
  reste est nulle ou non. 

  Le polynome generateur sera le CRC-CCITT : 
  
  X^16 + X^12 + X^5 + 1

  En base 10, ce nombre vaut 69665

\*================================================*/


#include "Detection.h"


void* AllongerDeDeuxOctets(void *objet, int taille)
/*===============================================*\
  Cette fonction peut peut-etre se reveler 
  totalement inutile selon les structures de 
  donnees qui sont utilisees. 
  Son objectif est de multiplier un polynome
  quelconque (objet) de degre taille-1 par 
  le polynome x^16. En d'autres termes, multiplier
  le nombre "objet" par 2^16. Autrement dit encore, 
  il s'agit d'ajouter 16 zeros au poids faible 
  du nombre objet. 
  Attention : Le fait d'avoir allonge l'objet
  de deux octets lui a peut-etre change son 
  adresse !!!
\*===============================================*/
{
  int i; 
  void *objet_copie;

  objet_copie = (void*)calloc(taille+2, 1); 
  if(objet_copie==NULL)
    {printf("AllongerDeDeuxOctets : calloc a echoue.\n");
    exit(1);}

  /* initialiser la valeur des deux nouveaux octets */
  for(i=0; i<16; i++)
    SetNiemeBit(objet_copie, taille+2, i, 0);

  if(objet)
    {
      memcpy(objet_copie, objet, taille); 
      free(objet);
    }
  return(objet_copie);
}


void* DividendeCopie(void *dividende, int taille)
/*===============================================*\
  Copie l'objet dividende dans un autre objet 
  et rend le pointeur. Cette fonction utilise 
  toujours le meme tableau tant que c'est 
  possible. Quand ce tableau se revele trop
  petit, alors il en prend un plus grand et 
  jette l'ancien. 
  Attention : cette fonction utilise donc une 
  variable statique designant cette copie. 
  Donc cette fonction ne doit etre appelee par 
  aucune autre fonction que ResteDivisionModulo2, 
  sinon son contenu pourrait tres bien changer des 
  que cette fonction est appelee de nouveau !!!
\*===============================================*/
{
  static void* dividende_copie=NULL;
  static int taille_prec=0; 

  if(taille<=taille_prec)
    memcpy(dividende_copie, dividende, taille);

  else
    {
      dividende_copie=calloc(taille, 1);  
      
      if(dividende_copie==NULL)
	{ fprintf(stderr,"Dividende_copie : calloc a echoue.\n");
	exit(1);}
      else
	{
	  taille_prec = taille; 
	  memcpy(dividende_copie, dividende, taille);
	  if(dividende) free(dividende);
	}
    }  
  return(dividende_copie);
}


short ResteDivisionModulo2(void *dividende, int taille, int diviseur)
/*===============================================*\
  Fait la division modulo 2 de dividende par 
  le polynome diviseur et rend le reste. 
  Le diviseur doit etre de degre strictement
  inferieur a 32. (En general le polynome 
  generateur est de degre <= 16)
\*===============================================*/
{
  int dividende_interm, reste; 
  int degredvs, degredvd, i; 
  char *dividende_copie; 

  degredvd = Degre(dividende, taille);
  degredvs = Degre(&diviseur, sizeof(int));

  if(degredvs>31) 
    printf("degredvs=%d>31 : risque de dysfonctionnement.\n");

  fprintf(stderr,"degredvd=%d\tdegredvs=%d\n", degredvd, degredvs);

  if(degredvd<degredvs)
    {memcpy(&reste, dividende, taille);
    DecalageDroite(&reste, 2, reste-degredvd);
    return(reste);}
  /* Si le degre de la dividende est plus petit que celui du */
  /* diviseur alors on rend la dividende en reste -----------*/

  dividende_copie = DividendeCopie(dividende, taille);
  /* dividende_copie a encore une taille a priori quelconque */
  /* on ne peut pas le mettre dans un int. Reste un void*    */

  CheckBitABit(dividende_copie, taille, "dividende_copie");

  DecalageDroite(dividende_copie, taille, degredvd-degredvs);
  /* Grace a ce decalage, dividende_copie a au plus un degre  */
  /* egal a degredvs<32, on peut le representer par un int    */
  /* d'ou l'instruction suivante qui n'a pas d'autre but.     */

  CheckBitABit(dividende_copie, taille, "dividende_copie");

  memcpy(&dividende_interm, dividende_copie, sizeof(int));
  /*  DecalageDroite(&dividende_interm, sizeof(int), 8*(sizeof(int)-taille));*/

  CheckBitABit(&dividende_interm, sizeof(int), "dividende_interm");

  /* Soustraction en modulo2 <=> ou exclusif */
  reste = dividende_interm ^ diviseur; 

  for(i=degredvd-degredvs; i>0; i--)
    {
      dividende_interm = reste<<1; 

      /* On descend le prochain chiffre de la dividende */
      SetNiemeBit(&dividende_interm, 
		  sizeof(int), 
		  0, 
		  NiemeBit(dividende, taille, i-1)); 

      if(Degre(&dividende_interm, sizeof(int))>=degredvs) 
	/*quotient : 1*/

	/* Soustraction par 1 fois le diviseur */
	reste = dividende_interm ^ diviseur;

      else
	/* Soustraction par rien  */
	reste = dividende_interm; 
    }
  return(reste);
}


