
/*===============================================*\
	Vendredi 11 Septembre 1998

	Arash HABIBI

	SocketToolbox.c

 	Un ensemble de fonctions qu'on utilise
	partout en manipulant les sockets.  
	
\*================================================*/


#include "SocketToolbox.h"


u_long IPAddrByName(char *Nom)
/*================================================*\
	Fonction qui, a partir du nom d'un hote, 
	fournit l'adresse IP (long sous format hote)
\*================================================*/
{
	u_long AdresseIP; 
	struct hostent *HostEnt; 
	
	HostEnt = gethostbyname(Nom); 
	if(HostEnt==0) {printf("Pas d'hote du nom de %s\n",Nom); exit(1);}
	
	memcpy(&AdresseIP, HostEnt->h_addr, HostEnt->h_length); 
		
	return(AdresseIP);
}



void LongToAdresseIP(u_long A_IP, AdresseIP_t *habituel)
/*================================================*\
	Fonction qui a partir d'un entier long
	(consideree comme une adresse IP au format
	hote) fournit les quatre entiers tels que 
	l'adresse IP est representee sour la forme
	habituelle de n1.n2.n3.n4
\*================================================*/
{
	int reste;
	
	habituel->n1 = A_IP / 0x1000000; 
	reste = A_IP % 0x1000000; 
		
	habituel->n2 = reste / 0x10000; 
	reste = reste % 0x10000; 
	
	habituel->n3 = reste / 0x100; 
	habituel->n4 = reste % 0x100; 
}



struct sockaddr_in WriteNetworkAddress(char *HostName, u_short port)
/*================================================*\
	A partir de donnees de l'hote prepare une
	adresse format reseau prete a etre envoyee. 
\*================================================*/
{
	struct sockaddr_in adresse_dist; 

	/*---- Si on ne s'occupe pas de la portabilite ---*/
	/*---- alors il suffit de faire ce qui suit ------*/
	
	/* adresse_dist.sin_family      = AF_INET;        */
	/* adresse_dist.sin_port        = port;           */
	/* adresse_dist.sin_addr.s_addr = 		  */
	/*			IPAddrByName(HostName);   */

	/*---- Si oui alors il faut ecrire ---------------*/

	adresse_dist.sin_family      = AF_INET;
	adresse_dist.sin_port        = htons(port);  
	adresse_dist.sin_addr.s_addr = IPAddrByName(HostName); 
	
	/* Ici il n'y a plus besoin de htonl car la fonction */
	/* gethostbyname donne des adresses IP en format reseaux */
	
	return(adresse_dist);
}



void ReadNetworkAddress(struct sockaddr_in adresse_dist, AdresseIP_t *IP, u_short *port)
/*================================================*\
	A partir d'une adresse format reseau, produit
	l'adresse format hote sous la forme habituelle
	de n1.n2.n3.n4
\*================================================*/
{
	/*---- Si on ne s'occupe pas de la portabilite ---*/
	/*---- alors il suffit de faire ce qui suit ------*/
	
	/* *port = adresse_dist.sin_port;           	  */
	/* LongToAdresseIP(adresse_dist.sin_addr.s_addr,  */
	/*	IP);			                  */

	/*---- Si oui alors il faut ecrire ---------------*/
	
	*port = ntohs(adresse_dist.sin_port);
	LongToAdresseIP(ntohl(adresse_dist.sin_addr.s_addr), IP);			  
}



int GetAndBindSocket(int type, short nb_port, struct sockaddr_in *adresse_loc)
/*================================================*\
	A partir d'un type de socket (SOCK_DGRAM ou 
	SOCK_STREAM) d'un numero de port, fournit
	une adresse (au format hote) de socket 
	locale prete a etre utilisee. 
\*================================================*/
{
	int desc, res, taille; 

	if(adresse_loc==NULL) return(-1); 
	
/* Il faut que adresse_loc soit deja allouee */

	desc = socket(AF_INET, type, 0);

	if(desc == -1) {perror("socket"); exit(1);}
	else printf("Socket creee.\n");
	
/*--- preparation de la structure adresse locale ---*/
/*--- sans s'occuper de la portabilite du code -----*/
		
	/* adresse_loc->sin_family      = AF_INET;    */
	/* adresse_loc->sin_port        = nb_port;    */
	/* adresse_loc->sin_addr.s_addr = INADDR_ANY; */
	
	
/*--- INADDR_ANY donne l'adresse IP de la ----------*/
/*--- machine locale. Ce qui suit : portable -------*/
	
	adresse_loc->sin_family      = AF_INET;
	adresse_loc->sin_port 	     = htons(nb_port);          
	adresse_loc->sin_addr.s_addr = htonl(INADDR_ANY);
	
	
/*---- Attachement effective de la socket locale ---*/

	res = bind(desc, (struct sockaddr *)adresse_loc, 
		sizeof(*adresse_loc));
	
	if(res == -1) {perror("bind"); exit(1);}
	else printf("Socket attachee.\n");
	
	
/*---- Recupere l'adresse effective de l'attachement ---*/
/*---- (pour le cas ou le numero de port n'a pas ete ---*/
/*---- explicitement specifie (0) ----------------------*/
/*---- getsockname donne une adresse au format reseau --*/
	
	taille = sizeof(*adresse_loc);
	
	res = getsockname(desc, (struct sockaddr *)adresse_loc, 
		&taille);
	
/* -- Pour avoir l'adresse sous format hote ----------- */
	
	adresse_loc->sin_port = ntohs(adresse_loc->sin_port);
	adresse_loc->sin_addr.s_addr = 
	       ntohl(adresse_loc->sin_addr.s_addr);

	return(desc);
}



